# coding=utf-8
from unittest import TestCase
from earthdiagnostics.cmormanager import CMORManager
from earthdiagnostics.modelingrealm import ModelingRealms
from mock import Mock
import tempfile
import os
import shutil


class TestCMORManager(TestCase):

    def setUp(self):
        self.config = Mock()
        self.config.data_convention = 'specs'
        self.config.data_type = 'exp'
        self.config.experiment.expid = 'expid'
        self.config.experiment.model = 'model'
        self.config.experiment.experiment_name = 'expname'
        self.config.experiment.institute = 'institute'
        self.config.experiment.member_count_start = 0
        self.config.experiment.atmos_timestep = 6
        self.config.experiment.ocean_timestep = 6
        self.config.experiment.chunk_size = 12
        self.config.experiment.calendar = 'standard'

        self.config.cmor.initialization_number = 1
        self.config.cmor.version = ''

        self.tmp_dir = tempfile.mkdtemp()
        os.mkdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        self.config.data_dir = self.tmp_dir

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_find_data(self):
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'expid'))

    def test_find_data_fail(self):
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        with self.assertRaises(Exception):
            CMORManager(self.config)

    def test_find_data_with_model(self):
        os.makedirs(os.path.join(self.tmp_dir, 'model', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'model', 'expid'))

    def test_find_data_with_ecearth_fix(self):
        self.config.experiment.model = 'EC-Earth'
        os.makedirs(os.path.join(self.tmp_dir, 'ecearth', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'ecearth', 'expid'))

    def test_find_data_with_type_and_model(self):
        os.makedirs(os.path.join(self.tmp_dir, 'exp', 'model', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'exp', 'model', 'expid'))

    def test_get_varfolder(self):
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.get_varfolder(ModelingRealms.ocean, 'var'),
                         'var_f6h')

    def test_get_file_path_specs(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_version(self):
        self.config.cmor.version = 'version'
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/version/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_grid(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency, 'grid')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/grid/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_year(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'year'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'year'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, year='1998')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/year/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_1998.nc'))

        frequency.frequency = 'other'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'other'
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', None, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998')

    def test_get_file_path_raise_incomaptible_date_info(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'monthly'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'

        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, date_str='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          None, frequency, year='1998', date_str='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998', date_str='1998')

    def test_get_file_path_specs_date_str(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        frequency.frequency = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, date_str='date_str')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_date_str.nc'))

